#!/usr/bin/env python
#
# Get Calibration Data from the TART telescope
#
# Copyright (c) Tim Molteno 2017-2021.

import argparse
import numpy as np
import time
import json


from tart.operation import settings
from tart.imaging import visibility
from tart.imaging import calibration

from tart_tools import api_handler
from tart_tools import api_imaging
from tart.util import utc


def split_param(x):
    rot_degrees = x[0]
    re = np.concatenate(([1], x[1:24]))
    im = np.concatenate(([0], x[24:47]))
    gains = np.sqrt(re * re + im * im)
    phase_offsets = np.arctan2(im, re)
    return rot_degrees, gains, phase_offsets


def load_data(api, config):
    print("Loading new data from {}...".format(api.root))
    vis_json = api.get("imaging/vis")
    ts = api_imaging.vis_json_timestamp(vis_json)

    cat_url = api.catalog_url(lon=config.get_lon(),
                                lat=config.get_lat(),
                                datestr=utc.to_string(ts))
    print(f"Getting catalog from {cat_url}")
    src_json = api.get_url(cat_url)

    print("Loading Complete")
    return vis_json, src_json


def main():
    PARSER = argparse.ArgumentParser(
        description="Generate Calibration Data.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    PARSER.add_argument(
        "--api",
        required=False,
        default="https://tart.elec.ac.nz/signal",
        help="Telescope API server URL.",
    )
    PARSER.add_argument(
        "--file",
        required=False,
        default="calibration_data.json",
        help="Calibration Output JSON file.",
    )
    PARSER.add_argument(
        "--n", type=int, default=5, help="Number of samples in the JSON data file."
    )
    PARSER.add_argument(
        "--interval",
        type=int,
        default=5,
        help="Interval (in minutes) between samples in the JSON data file.",
    )
    PARSER.add_argument(
        "--catalog",
        required=False,
        default="https://tart.elec.ac.nz/catalog",
        help="Catalog API URL.",
    )

    ARGS = PARSER.parse_args()

    api = api_handler.APIhandler(ARGS.api)

    info = api.get("info")
    ant_pos = api.get("imaging/antenna_positions")
    config = settings.from_api_json(info["info"], ant_pos)
    gains_json = api.get("calibration/gain")

    ret = {"info": info, "ant_pos": ant_pos, "gains": gains_json}
    data = []
    for i in range(ARGS.n):
        vis_json, src_json = load_data(api, config)
        data.append([vis_json, src_json])
        if i != ARGS.n - 1:
            print("Sleeping {} minutes".format(ARGS.interval))
            time.sleep(ARGS.interval * 60)
    ret["data"] = data
    with open(ARGS.file, "w") as fp:
        json.dump(ret, fp, indent=4, separators=(",", ": "))



#!/usr/bin/env python
#
# Generate corresponding visibility (json) files from *.vis files
# Max Scheel 2017 tim@elec.ac.nz
#

import argparse
import json

from tart.imaging import visibility
from tart.util import utc

def create_direct_vis_dict(vis):
    """ This function is provided with a visibility object. It returns a dictionary identical to the json response. """
    vis_dicts = []
    for visobj in vis["vis_list"]:
        vis_list = []
        timestamp = utc.to_string(visobj.timestamp)
        for b, v in zip(visobj.baselines, visobj.v):
            i, j = b
            vis_el = {"i": i, "j": j, "re": v.real, "im": v.imag}
            vis_list.append(vis_el)
        vis_dicts.append({"data": [({"data": vis_list, "timestamp": timestamp},
                                    [])],
                          "timestamp": timestamp})
    return vis_dicts


def main():
    PARSER = argparse.ArgumentParser(
        description="Generate offline json files from a list of visibility objects files."
    )
    PARSER.add_argument(
        "--vis", required=False, nargs="*", default="", help="Visibilities data file."
    )
    ARGS = PARSER.parse_args()
    VIS_LIST = []
    if len(ARGS.vis) != 0:
        for vis_file in ARGS.vis:
            VIS_LIST.append(visibility.list_load(vis_file))

    for vis in VIS_LIST:
        dico_info = {"info": json.loads(vis['config'].to_json())}
        dico_info["info"]['operating_frequency'] = dico_info["info"]['frequency']
        dico_info["info"]['location'] = {"lat": dico_info["info"]["lat"],
                                         "lon": dico_info["info"]["lon"],
                                         "alt": dico_info["info"]["alt"]}
        vis_dicts = create_direct_vis_dict(vis)
        for vis_dict in vis_dicts:
            vis_dict["info"] = dico_info
            vis_dict["ant_pos"] = vis["ant_pos"].tolist()
            vis_dict["gains"] = {"gain": vis["gain"].tolist(),
                                 "phase_offset": vis["phase_offset"].tolist()}
            fname = "vis_" + vis_dict["timestamp"] + ".json"
            with open(fname, "w") as fp:
                json.dump(vis_dict, fp)



#!/usr/bin/env python
#    Save gains from api to local file.
#    Max Scheel 2017 - max@max.ac.nz
#    Tim Molteno 2017 - tim@elec.ac.nz

import argparse
import json

from tart_tools.api_handler import APIhandler, download_current_gain

def main():
    parser = argparse.ArgumentParser(
        description="Save gains from api to local file",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "--api",
        required=False,
        default="https://tart.elec.ac.nz/signal",
        help="Telescope API server URL.",
    )
    parser.add_argument(
        "--file", default="gains.json", type=str, help="local file to dump gains"
    )

    ARGS = parser.parse_args()

    api = APIhandler(ARGS.api)

    gains = download_current_gain(api)

    with open(ARGS.file, "w") as outfile:
        json.dump(gains, outfile, sort_keys=True, indent=4, ensure_ascii=False)



"""
Test the AuthorizedAPIhandler

"""
#
# Copyright (c) Tim Molteno 2023. tim@elec.ac.nz
#


import time
import unittest

from tart_tools.api_handler import AuthorizedAPIhandler


class TestApi(unittest.TestCase):

    def setUp(self):
        self.api = 'https://api.elec.ac.nz/tart/nz-dunedin'
        self.pw = 'sharkbait'

    def test_auth(self):
        # Test the expiration of the auth token and that it is refreshed correctly
        api = AuthorizedAPIhandler(self.api, self.pw)
        resp = api.post_with_token("mode/vis")
        for i in range(11):
            time.sleep(120)
            resp = api.post_with_token("mode/vis")
            print(f"response {resp}")

        self.assertAlmostEqual(resp, 0)
